{% extends "base.html" %}

{% load feincms3 %}

{% block title %}{{ page }} - {{ block.super }}{% endblock %}

{% block content %}
<h1>{{ page }}</h1>
<div class="columns">
  <div class="main">{% render_region regions 'main' %}</div>
  <div class="sidebar">{% render_region regions 'sidebar' %}</div>
</div>
{% endblock %}
