<!DOCTYPE HTML>
<html>
<head>
  <title>{% block title %}testapp{% endblock %}</title>
</head>
<body>
  {% if messages %}
  <ul class="messages">
      {% for message in messages %}
      <li{% if message.tags %} class="{{ message.tags }}"{% endif %}>{{ message }}</li>
      {% endfor %}
  </ul>
  {% endif %}
  {% block content %}{% endblock %}
</body>
</html>
