{% extends "base.html" %}

{% load feincms3 %}

{% block title %}{{ page }} - {{ block.super }}{% endblock %}

{% block content %}
<h1>{{ page }}</h1>
{% render_region regions 'main' %}
{% endblock %}
